@@ -20,6 +20,7 @@ import hashlib
 
 from collections import OrderedDict
 from typing import List, Text, Dict, Any
+from os import environ as env
 
 from kfp import dsl
 from kfp_tekton.compiler._k8s_helper import convert_k8s_obj_to_json, sanitize_k8s_name
@@ -564,6 +565,8 @@ def _op_to_template(op: BaseOp,
         }
         template.setdefault('metadata', {}).setdefault('annotations', {})['pipelines.kubeflow.org/component_spec_digest'] = \
             json.dumps(digested_component_spec_dict, sort_keys=True)
+        if env.get('DISABLE_ARTIFACT_TRACKING', 'false').lower() == 'true':
+            template['metadata']['annotations'].pop('pipelines.kubeflow.org/component_spec_digest', None)
 
     if isinstance(op, dsl.ContainerOp) and op.execution_options:
         if op.execution_options.caching_strategy.max_cache_staleness:
@@ -579,7 +582,7 @@ def _op_to_template(op: BaseOp,
         try:
             result_size_map = json.loads(result_size_map)
         except ValueError:
-            raise("tekton-result-sizes annotation is not a valid JSON")
+            raise ("tekton-result-sizes annotation is not a valid JSON")
         # Normalize estimated result size keys.
         result_size_map = {sanitize_k8s_name(key, allow_capital_underscore=True): value
                            for key, value in result_size_map.items()}
@@ -596,7 +599,7 @@ def _op_to_template(op: BaseOp,
             try:
                 value = int(value)
             except ValueError:
-                raise("Estimated value for result %s is %s, but it needs to be an integer." % (key, value))
+                raise ("Estimated value for result %s is %s, but it needs to be an integer." % (key, value))
             if key in op_result_names:
                 packed_index = -1
                 # Look for bin that can fit the result value
