@@ -74,16 +74,16 @@ def _handle_tekton_pipeline_variables(pipeline_run):
     }
 
     add_type = pipeline_run.get('kind') != 'PipelineRun'
-    task_list = pipeline_run['spec']['pipelineSpec']['tasks']
-    for task in task_list:
+
+    def handle_task(task):
         if task.get('taskRef', {}):
-            continue
+            return
         if 'taskSpec' in task and 'apiVersion' in task['taskSpec']:
             # recur for embedded pipeline resources
             if 'spec' in task['taskSpec'] and 'pipelineSpec' in task['taskSpec']['spec']:
                 resource_spec = task['taskSpec']
                 _handle_tekton_pipeline_variables(resource_spec)
-            continue
+            return
 
         for key, val in pipeline_variables.items():
             task_str = json.dumps(task['taskSpec']['steps'])
@@ -108,6 +108,13 @@ def _handle_tekton_pipeline_variables(pipeline_run):
                         param['type'] = 'string'
                     task['taskSpec'].setdefault('params', []).append(param)
 
+    task_list = pipeline_run.get('spec', {}).get('pipelineSpec', {}).get('tasks', [])
+    finally_task_list = pipeline_run.get('spec', {}).get('pipelineSpec', {}).get('finally', [])
+    for task in task_list:
+        handle_task(task)
+    for task in finally_task_list:
+        handle_task(task)
+
     return pipeline_run
 
 
@@ -380,6 +387,13 @@ def _handle_tekton_custom_task(custom_task: dict, workflow: dict, recursive_task
                             break
                     if not has_nested_task:
                         break
+                nested_loop_counter_params = []
+                # Fetch nested loop counter params so that it won't find the nested parameters from
+                # global param level.
+                for task in custom_task_crs:
+                    if task['metadata']['name'] in all_nested_loop:
+                        if task['spec'].get('iterateNumeric'):
+                            nested_loop_counter_params.append(task['spec'].get('iterateNumeric'))
                 for task in tasks:
                     if task['name'] == nested_custom_task['root_ct']:
                         task['params'].extend(copy.deepcopy(nested_custom_task_special_params))
@@ -390,7 +404,8 @@ def _handle_tekton_custom_task(custom_task: dict, workflow: dict, recursive_task
                         task['params'] = sorted(task['params'], key=lambda k: k['name'])
                         if task['name'] in all_nested_loop:
                             for param in task['params']:
-                                if '$(params.' in param['value'] and 'subvar-' not in param['value']:
+                                if '$(params.' in param['value'] and 'subvar-' not in param['value'] and \
+                                  param['name'] not in nested_loop_counter_params:
                                     global_task_values.add(param['value'])
                 # Add any pipeline global params to the nested loop layers
                 all_params = []
